==> 4.7.4.tar.gz: terraform-provider-decort/internal/service/cloudbroker/image/resource_image.go <==
/*
Copyright (c) 2019-2023 Digital Energy Cloud Solutions LLC. All Rights Reserved.
Authors:
Petr Krutov, <petr.krutov@digitalenergy.online>
Stanislav Solovev, <spsolovev@digitalenergy.online>

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

/*
Terraform DECORT provider - manage resources provided by DECORT (Digital Energy Cloud
Orchestration Technology) with Terraform by Hashicorp.

Source code: https://repository.basistech.ru/BASIS/terraform-provider-decort

Please see README.md to learn where to place source code so that it
builds seamlessly.

Documentation: https://repository.basistech.ru/BASIS/terraform-provider-decort/wiki
*/

package image

import (
	"context"
	"strconv"

	"github.com/hashicorp/terraform-plugin-sdk/v2/diag"
	"github.com/hashicorp/terraform-plugin-sdk/v2/helper/schema"
	log "github.com/sirupsen/logrus"
	"repository.basistech.ru/BASIS/decort-golang-sdk/pkg/cloudbroker/image"
	"repository.basistech.ru/BASIS/terraform-provider-decort/internal/constants"
	"repository.basistech.ru/BASIS/terraform-provider-decort/internal/controller"
	"repository.basistech.ru/BASIS/terraform-provider-decort/internal/dc"
	"repository.basistech.ru/BASIS/terraform-provider-decort/internal/status"
)

func resourceImageCreate(ctx context.Context, d *schema.ResourceData, m interface{}) diag.Diagnostics {
	log.Debugf("resourceImageCreate: called for image %s", d.Get("name").(string))

	url, urlOk := d.GetOk("url")
	filePath, fileImageOk := d.GetOk("file_path")

	if !urlOk && !fileImageOk {
		return diag.Errorf("resourceImageCreate: no valid url or file_path specified")
	}

	if urlOk && fileImageOk {
		return diag.Errorf("resourceImageCreate: either url or resource file_path should be specified")
	}

	c := m.(*controller.ControllerCfg)

	syncMode := d.Get("sync_mode").(bool)
	var imageId uint64

	if fileImageOk {
		var err error
		url, err = c.CloudBroker().Image().UploadImageFile(ctx, filePath.(string))
		if err != nil {
			return diag.FromErr(err)
		}
	}

	if syncMode {
		req, err := SyncCreateRequest(ctx, d, m, url.(string))
		if err != nil {
			return diag.FromErr(err)
		}
		imageId, err = c.CloudBroker().Image().SyncCreate(ctx, req)
		if err != nil {
			return diag.FromErr(err)
		}
	} else {
		req, err := CreateRequest(ctx, d, m, url.(string))
		if err != nil {
			return diag.FromErr(err)
		}
		imageId, err = c.CloudBroker().Image().CreateImage(ctx, req)
		if err != nil {
			return diag.FromErr(err)
		}
	}

	d.SetId(strconv.FormatUint(imageId, 10))

	var w dc.Warnings

	if _, ok := d.GetOk("accounts"); ok {
		if err := resourceImageSetAccess(ctx, d, m); err != nil {
			w.Add(err)
		}
	}

	return append(resourceImageRead(ctx, d, m), w.Get()...)
}

func resourceImageRead(ctx context.Context, d *schema.ResourceData, m interface{}) diag.Diagnostics {
	log.Debugf("resourceImageRead: called for %s id: %s", d.Get("name").(string), d.Id())

	img, err := utilityImageCheckPresence(ctx, d, m)
	if err != nil {
		d.SetId("")
		return diag.FromErr(err)
	}

	switch img.Status {
	case status.Modeled:
		return diag.Errorf("The image is in status: %s, please, contact support for more information", img.Status)
	case status.Destroyed, status.Purged:
		d.SetId("")
		return diag.Errorf("The resource cannot be updated because it has been destroyed")
	}

	flattenImage(d, img)

	return nil
}

func resourceImageDelete(ctx context.Context, d *schema.ResourceData, m interface{}) diag.Diagnostics {
	log.Debugf("resourceImageDelete: called for %s, id: %s", d.Get("name").(string), d.Id())

	_, err := utilityImageCheckPresence(ctx, d, m)
	if err != nil {
		d.SetId("")
		return diag.FromErr(err)
	}

	c := m.(*controller.ControllerCfg)
	req := image.DeleteRequest{
		ImageID: uint64(d.Get("image_id").(int)),
	}

	_, err = c.CloudBroker().Image().Delete(ctx, req)
	if err != nil {
		return diag.FromErr(err)
	}

	d.SetId("")

	return nil
}

func resourceImageUpdate(ctx context.Context, d *schema.ResourceData, m interface{}) diag.Diagnostics {
	log.Debugf("resourceImageEdit: called for %s, id: %s", d.Get("name").(string), d.Id())

	img, err := utilityImageCheckPresence(ctx, d, m)
	if err != nil {
		d.SetId("")
		return diag.FromErr(err)
	}

	switch img.Status {
	case status.Modeled:
		return diag.Errorf("The image is in status: %s, please, contact support for more information", img.Status)
	case status.Destroyed, status.Purged:
		d.SetId("")
		return diag.Errorf("The resource cannot be updated because it has been destroyed")
	}

	if d.HasChange("enabled") {
		err := resourceImageChangeEnabled(ctx, d, m)
		if err != nil {
			return diag.FromErr(err)
		}
	}

	if d.HasChange("shared_with") {
		err := resourceImageShare(ctx, d, m)
		if err != nil {
			return diag.FromErr(err)
		}
	}

	if d.HasChange("computeci_id") {
		err := resourceImageChangeComputeci(ctx, d, m)
		if err != nil {
			return diag.FromErr(err)
		}
	}

	if d.HasChange("enabled_stacks") {
		err := resourceImageUpdateNodes(ctx, d, m)
		if err != nil {
			return diag.FromErr(err)
		}
	}

	if d.HasChanges("name", "username", "password", "account_id", "bootable", "hot_resize", "network_interface_naming") {
		err := resourceImageEdit(ctx, d, m)
		if err != nil {
			return diag.FromErr(err)
		}
	}

	if d.HasChange("accounts") {
		err := resourceImageChangeAccess(ctx, d, m)
		if err != nil {
			return diag.FromErr(err)
		}
	}

	return nil
}

func resourceImageChangeEnabled(ctx context.Context, d *schema.ResourceData, m interface{}) error {
	c := m.(*controller.ControllerCfg)
	imageId := uint64(d.Get("image_id").(int))

	if d.Get("enabled").(bool) {
		req := image.EnableRequest{
			ImageID: imageId,
		}

		_, err := c.CloudBroker().Image().Enable(ctx, req)
		if err != nil {
			return err
		}
	} else {
		req := image.DisableRequest{
			ImageID: imageId,
		}

		_, err := c.CloudBroker().Image().Disable(ctx, req)
		if err != nil {
			return err
		}
	}

	return nil
}

func resourceImageShare(ctx context.Context, d *schema.ResourceData, m interface{}) error {
	log.Debugf("resourceImageShare: called for %s, id: %s", d.Get("name").(string), d.Id())
	c := m.(*controller.ControllerCfg)
	req := image.ShareRequest{
		ImageId: uint64(d.Get("image_id").(int)),
	}
	accIds := []uint64{}
	for _, accId := range d.Get("shared_with").([]interface{}) {
		accIds = append(accIds, uint64(accId.(int)))
	}
	req.AccountIDs = accIds

	_, err := c.CloudBroker().Image().Share(ctx, req)
	if err != nil {
		return err
	}

	return nil
}

func resourceImageChangeComputeci(ctx context.Context, d *schema.ResourceData, m interface{}) error {
	c := m.(*controller.ControllerCfg)

	imageId := uint64(d.Get("image_id").(int))
	computeci := uint64(d.Get("computeci_id").(int))

	if computeci == 0 {
		req := image.ComputeCIUnsetRequest{
			ImageID: imageId,
		}

		_, err := c.CloudBroker().Image().ComputeCIUnset(ctx, req)
		if err != nil {
			return err
		}
	} else {
		req := image.ComputeCISetRequest{
			ImageID:     imageId,
			ComputeCIID: computeci,
		}

		_, err := c.CloudBroker().Image().ComputeCISet(ctx, req)
		if err != nil {
			return err
		}
	}

	return nil
}

func resourceImageUpdateNodes(ctx context.Context, d *schema.ResourceData, m interface{}) error {
	log.Debugf("resourceImageUpdateNodes: called for %s, id: %s", d.Get("name").(string), d.Id())
	c := m.(*controller.ControllerCfg)
	req := image.UpdateNodesRequest{
		ImageID: uint64(d.Get("image_id").(int)),
	}
	enabledStacks := []uint64{}
	for _, stack := range d.Get("enabled_stacks").([]interface{}) {
		enabledStacks = append(enabledStacks, uint64(stack.(int)))
	}

	req.EnabledStacks = enabledStacks

	_, err := c.CloudBroker().Image().UpdateNodes(ctx, req)
	if err != nil {
		return err
	}

	return nil
}

func resourceImageEdit(ctx context.Context, d *schema.ResourceData, m interface{}) error {
	log.Debugf("resourceImageEdit: called for %s, id: %s", d.Get("name").(string), d.Id())
	c := m.(*controller.ControllerCfg)
	req := image.EditRequest{}

	req.ImageID = uint64(d.Get("image_id").(int))
	if d.HasChange("name") {
		req.Name = d.Get("name").(string)
	}
	if d.HasChange("username") {
		req.Username = d.Get("username").(string)
	}
	if d.HasChange("password") {
		req.Password = d.Get("password").(string)
	}
	if d.HasChange("account_id") {
		req.AccountID = uint64(d.Get("account_id").(int))
	}
	if d.HasChange("bootable") {
		req.Bootable = d.Get("bootable").(bool)
	}
	if d.HasChange("hot_resize") {
		req.HotResize = d.Get("hot_resize").(bool)
	}
	if d.HasChange("network_interface_naming") {
		req.NetworkInterfaceNaming = d.Get("network_interface_naming").(string)
	}
	_, err := c.CloudBroker().Image().Edit(ctx, req)
	if err != nil {
		return err
	}
	return nil
}

func resourceImageChangeAccess(ctx context.Context, d *schema.ResourceData, m interface{}) error {
	log.Debugf("resourceImageChangeAccess: called for %s, id: %s", d.Get("name").(string), d.Id())
	c := m.(*controller.ControllerCfg)

	imageId, _ := strconv.ParseUint(d.Id(), 10, 64)

	oldSet, newSet := d.GetChange("accounts")

	revokeAccess := (oldSet.(*schema.Set).Difference(newSet.(*schema.Set))).List()
	if len(revokeAccess) > 0 {
		revokeAccounts := make([]uint64, 0, len(revokeAccess))
		for _, accountId := range revokeAccess {
			revokeAccounts = append(revokeAccounts, uint64(accountId.(int)))
		}

		req := image.RevokeAccessRequest{
			ImageID:    imageId,
			AccountIDs: revokeAccounts,
		}

		_, err := c.CloudBroker().Image().RevokeAccess(ctx, req)
		if err != nil {
			return err
		}
	}

	addedAccess := (newSet.(*schema.Set).Difference(oldSet.(*schema.Set))).List()
	if len(addedAccess) > 0 {
		grantAccounts := make([]uint64, 0, len(addedAccess))
		for _, accountId := range addedAccess {
			grantAccounts = append(grantAccounts, uint64(accountId.(int)))
		}

		req := image.GrantAccessRequest{
			ImageID:    imageId,
			AccountIDs: grantAccounts,
		}

		_, err := c.CloudBroker().Image().GrantAccess(ctx, req)
		if err != nil {
			return err
		}
	}

	return nil
}

func resourceImageSetAccess(ctx context.Context, d *schema.ResourceData, m interface{}) error {
	log.Debugf("resourceImageSetAccess: called for %s, id: %s", d.Get("name").(string), d.Id())
	c := m.(*controller.ControllerCfg)

	imageId, _ := strconv.ParseUint(d.Id(), 10, 64)

	accounts := d.Get("accounts").([]interface{})
	accountIDs := make([]uint64, 0, len(accounts))
	for _, accountId := range accounts {
		accountIDs = append(accountIDs, uint64(accountId.(int)))
	}

	req := image.GrantAccessRequest{
		ImageID:    imageId,
		AccountIDs: accountIDs,
	}

	_, err := c.CloudBroker().Image().GrantAccess(ctx, req)
	if err != nil {
		return err
	}

	return nil
}

func ResourceImage() *schema.Resource {
	return &schema.Resource{
		SchemaVersion: 1,

		CreateContext: resourceImageCreate,
		ReadContext:   resourceImageRead,
		UpdateContext: resourceImageUpdate,
		DeleteContext: resourceImageDelete,

		Importer: &schema.ResourceImporter{
			StateContext: schema.ImportStatePassthroughContext,
		},

		Timeouts: &schema.ResourceTimeout{
			Create:  &constants.Timeout600s,
			Read:    &constants.Timeout300s,
			Update:  &constants.Timeout300s,
			Delete:  &constants.Timeout300s,
			Default: &constants.Timeout300s,
		},

		Schema: resourceImageSchemaMake(),
	}
}

==> 4.9.0.tar.gz: terraform-provider-decort/internal/service/cloudbroker/image/resource_image.go <==
/*
Copyright (c) 2019-2023 Digital Energy Cloud Solutions LLC. All Rights Reserved.
Authors:
Petr Krutov, <petr.krutov@digitalenergy.online>
Stanislav Solovev, <spsolovev@digitalenergy.online>

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

/*
Terraform DECORT provider - manage resources provided by DECORT (Digital Energy Cloud
Orchestration Technology) with Terraform by Hashicorp.

Source code: https://repository.basistech.ru/BASIS/terraform-provider-decort

Please see README.md to learn where to place source code so that it
builds seamlessly.

Documentation: https://repository.basistech.ru/BASIS/terraform-provider-decort/wiki
*/

package image

import (
	"context"
	"strconv"

	"github.com/hashicorp/terraform-plugin-sdk/v2/diag"
	"github.com/hashicorp/terraform-plugin-sdk/v2/helper/schema"
	log "github.com/sirupsen/logrus"
	"repository.basistech.ru/BASIS/decort-golang-sdk/pkg/cloudbroker/image"
	"repository.basistech.ru/BASIS/terraform-provider-decort/internal/constants"
	"repository.basistech.ru/BASIS/terraform-provider-decort/internal/controller"
	"repository.basistech.ru/BASIS/terraform-provider-decort/internal/dc"
	"repository.basistech.ru/BASIS/terraform-provider-decort/internal/status"
)

func resourceImageCreate(ctx context.Context, d *schema.ResourceData, m interface{}) diag.Diagnostics {
	log.Debugf("resourceImageCreate: called for image %s", d.Get("name").(string))

	url, urlOk := d.GetOk("url")
	filePath, fileImageOk := d.GetOk("file_path")

	if !urlOk && !fileImageOk {
		return diag.Errorf("resourceImageCreate: no valid url or file_path specified")
	}

	if urlOk && fileImageOk {
		return diag.Errorf("resourceImageCreate: either url or resource file_path should be specified")
	}

	c := m.(*controller.ControllerCfg)

	syncMode := d.Get("sync_mode").(bool)
	var imageId uint64

	if fileImageOk {
		var err error
		url, err = c.CloudBroker().Image().UploadImageFile(ctx, filePath.(string))
		if err != nil {
			return diag.FromErr(err)
		}
	}

	if syncMode {
		req, err := SyncCreateRequest(ctx, d, m, url.(string))
		if err != nil {
			return diag.FromErr(err)
		}
		imageId, err = c.CloudBroker().Image().SyncCreate(ctx, req)
		if err != nil {
			return diag.FromErr(err)
		}
	} else {
		req, err := CreateRequest(ctx, d, m, url.(string))
		if err != nil {
			return diag.FromErr(err)
		}
		imageId, err = c.CloudBroker().Image().CreateImage(ctx, req)
		if err != nil {
			return diag.FromErr(err)
		}
	}

	d.SetId(strconv.FormatUint(imageId, 10))

	var w dc.Warnings

	if _, ok := d.GetOk("accounts"); ok {
		if err := resourceImageSetAccess(ctx, d, m); err != nil {
			w.Add(err)
		}
	}

	return append(resourceImageRead(ctx, d, m), w.Get()...)
}

func resourceImageRead(ctx context.Context, d *schema.ResourceData, m interface{}) diag.Diagnostics {
	log.Debugf("resourceImageRead: called for %s id: %s", d.Get("name").(string), d.Id())

	img, err := utilityImageCheckPresence(ctx, d, m)
	if err != nil {
		d.SetId("")
		return diag.FromErr(err)
	}

	switch img.Status {
	case status.Modeled:
		return diag.Errorf("The image is in status: %s, please, contact support for more information", img.Status)
	case status.Destroyed, status.Purged:
		d.SetId("")
		return diag.Errorf("The resource cannot be updated because it has been destroyed")
	}

	flattenImage(d, img)

	return nil
}

func resourceImageDelete(ctx context.Context, d *schema.ResourceData, m interface{}) diag.Diagnostics {
	log.Debugf("resourceImageDelete: called for %s, id: %s", d.Get("name").(string), d.Id())

	_, err := utilityImageCheckPresence(ctx, d, m)
	if err != nil {
		d.SetId("")
		return diag.FromErr(err)
	}

	c := m.(*controller.ControllerCfg)
	req := image.DeleteRequest{
		ImageID: uint64(d.Get("image_id").(int)),
	}

	_, err = c.CloudBroker().Image().Delete(ctx, req)
	if err != nil {
		return diag.FromErr(err)
	}

	d.SetId("")

	return nil
}

func resourceImageUpdate(ctx context.Context, d *schema.ResourceData, m interface{}) diag.Diagnostics {
	log.Debugf("resourceImageEdit: called for %s, id: %s", d.Get("name").(string), d.Id())

	img, err := utilityImageCheckPresence(ctx, d, m)
	if err != nil {
		d.SetId("")
		return diag.FromErr(err)
	}

	switch img.Status {
	case status.Modeled:
		return diag.Errorf("The image is in status: %s, please, contact support for more information", img.Status)
	case status.Destroyed, status.Purged:
		d.SetId("")
		return diag.Errorf("The resource cannot be updated because it has been destroyed")
	}

	if d.HasChange("enabled") {
		err := resourceImageChangeEnabled(ctx, d, m)
		if err != nil {
			return diag.FromErr(err)
		}
	}

	if d.HasChange("shared_with") {
		err := resourceImageShare(ctx, d, m)
		if err != nil {
			return diag.FromErr(err)
		}
	}

	if d.HasChange("computeci_id") {
		err := resourceImageChangeComputeci(ctx, d, m)
		if err != nil {
			return diag.FromErr(err)
		}
	}

	if d.HasChange("enabled_stacks") {
		err := resourceImageUpdateNodes(ctx, d, m)
		if err != nil {
			return diag.FromErr(err)
		}
	}

	if d.HasChanges("name", "username", "password", "account_id", "bootable", "hot_resize", "network_interface_naming") {
		err := resourceImageEdit(ctx, d, m)
		if err != nil {
			return diag.FromErr(err)
		}
	}

	if d.HasChange("accounts") {
		err := resourceImageChangeAccess(ctx, d, m)
		if err != nil {
			return diag.FromErr(err)
		}
	}

	return nil
}

func resourceImageChangeEnabled(ctx context.Context, d *schema.ResourceData, m interface{}) error {
	c := m.(*controller.ControllerCfg)
	imageId := uint64(d.Get("image_id").(int))

	if d.Get("enabled").(bool) {
		req := image.EnableRequest{
			ImageID: imageId,
		}

		_, err := c.CloudBroker().Image().Enable(ctx, req)
		if err != nil {
			return err
		}
	} else {
		req := image.DisableRequest{
			ImageID: imageId,
		}

		_, err := c.CloudBroker().Image().Disable(ctx, req)
		if err != nil {
			return err
		}
	}

	return nil
}

func resourceImageShare(ctx context.Context, d *schema.ResourceData, m interface{}) error {
	log.Debugf("resourceImageShare: called for %s, id: %s", d.Get("name").(string), d.Id())
	c := m.(*controller.ControllerCfg)
	req := image.ShareRequest{
		ImageId: uint64(d.Get("image_id").(int)),
	}
	accIds := []uint64{}
	for _, accId := range d.Get("shared_with").([]interface{}) {
		accIds = append(accIds, uint64(accId.(int)))
	}
	req.AccountIDs = accIds

	_, err := c.CloudBroker().Image().Share(ctx, req)
	if err != nil {
		return err
	}

	return nil
}

func resourceImageChangeComputeci(ctx context.Context, d *schema.ResourceData, m interface{}) error {
	c := m.(*controller.ControllerCfg)

	imageId := uint64(d.Get("image_id").(int))
	computeci := uint64(d.Get("computeci_id").(int))

	if computeci == 0 {
		req := image.ComputeCIUnsetRequest{
			ImageID: imageId,
		}

		_, err := c.CloudBroker().Image().ComputeCIUnset(ctx, req)
		if err != nil {
			return err
		}
	} else {
		req := image.ComputeCISetRequest{
			ImageID:     imageId,
			ComputeCIID: computeci,
		}

		_, err := c.CloudBroker().Image().ComputeCISet(ctx, req)
		if err != nil {
			return err
		}
	}

	return nil
}

func resourceImageUpdateNodes(ctx context.Context, d *schema.ResourceData, m interface{}) error {
	log.Debugf("resourceImageUpdateNodes: called for %s, id: %s", d.Get("name").(string), d.Id())
	c := m.(*controller.ControllerCfg)
	req := image.UpdateNodesRequest{
		ImageID: uint64(d.Get("image_id").(int)),
	}
	enabledStacks := []uint64{}
	for _, stack := range d.Get("enabled_stacks").([]interface{}) {
		enabledStacks = append(enabledStacks, uint64(stack.(int)))
	}

	req.EnabledStacks = enabledStacks

	_, err := c.CloudBroker().Image().UpdateNodes(ctx, req)
	if err != nil {
		return err
	}

	return nil
}

func resourceImageEdit(ctx context.Context, d *schema.ResourceData, m interface{}) error {
	log.Debugf("resourceImageEdit: called for %s, id: %s", d.Get("name").(string), d.Id())
	c := m.(*controller.ControllerCfg)
	req := image.EditRequest{}

	req.ImageID = uint64(d.Get("image_id").(int))
	if d.HasChange("name") {
		req.Name = d.Get("name").(string)
	}
	if d.HasChange("username") {
		req.Username = d.Get("username").(string)
	}
	if d.HasChange("password") {
		req.Password = d.Get("password").(string)
	}
	if d.HasChange("account_id") {
		req.AccountID = uint64(d.Get("account_id").(int))
	}
	if d.HasChange("bootable") {
		req.Bootable = d.Get("bootable").(bool)
	}
	if d.HasChange("hot_resize") {
		req.HotResize = d.Get("hot_resize").(bool)
	}
	if d.HasChange("network_interface_naming") {
		req.NetworkInterfaceNaming = d.Get("network_interface_naming").(string)
	}
	_, err := c.CloudBroker().Image().Edit(ctx, req)
	if err != nil {
		return err
	}
	return nil
}

func resourceImageChangeAccess(ctx context.Context, d *schema.ResourceData, m interface{}) error {
	log.Debugf("resourceImageChangeAccess: called for %s, id: %s", d.Get("name").(string), d.Id())
	c := m.(*controller.ControllerCfg)

	imageId, _ := strconv.ParseUint(d.Id(), 10, 64)

	oldSet, newSet := d.GetChange("accounts")

	revokeAccess := (oldSet.(*schema.Set).Difference(newSet.(*schema.Set))).List()
	revokeAccounts := make([]int64, 0, len(revokeAccess))
	if newSet.(*schema.Set).Len() == 0 {
		revokeAccounts = append(revokeAccounts, -1)
	} else if len(revokeAccess) > 0 {
		for _, accountId := range revokeAccess {
			revokeAccounts = append(revokeAccounts, accountId.(int64))
		}

	}

	req := image.RevokeAccessRequest{
		ImageID:    imageId,
		AccountIDs: revokeAccounts,
	}

	_, err := c.CloudBroker().Image().RevokeAccess(ctx, req)
	if err != nil {
		return err
	}

	addedAccess := (newSet.(*schema.Set).Difference(oldSet.(*schema.Set))).List()
	grantAccounts := make([]int64, 0, len(addedAccess))
	if newSet.(*schema.Set).Len() == 0 {
		grantAccounts = append(grantAccounts, -1)
	} else if len(addedAccess) > 0 {
		for _, accountId := range addedAccess {
			grantAccounts = append(grantAccounts, accountId.(int64))
		}

	}

	// req := image.GrantAccessRequest{
	// 	ImageID:    imageId,
	// 	AccountIDs: grantAccounts,
	// }

	// _, err := c.CloudBroker().Image().GrantAccess(ctx, req)
	// if err != nil {
	// 	return err
	// }

	return nil
}

func resourceImageSetAccess(ctx context.Context, d *schema.ResourceData, m interface{}) error {
	log.Debugf("resourceImageSetAccess: called for %s, id: %s", d.Get("name").(string), d.Id())
	c := m.(*controller.ControllerCfg)

	imageId, _ := strconv.ParseUint(d.Id(), 10, 64)

	accounts := d.Get("accounts").([]interface{})
	accountIDs := make([]int64, 0, len(accounts))
	for _, accountId := range accounts {
		accountIDs = append(accountIDs, accountId.(int64))
	}

	req := image.GrantAccessRequest{
		ImageID:    imageId,
		AccountIDs: accountIDs,
	}

	_, err := c.CloudBroker().Image().GrantAccess(ctx, req)
	if err != nil {
		return err
	}

	return nil
}

func ResourceImage() *schema.Resource {
	return &schema.Resource{
		SchemaVersion: 1,

		CreateContext: resourceImageCreate,
		ReadContext:   resourceImageRead,
		UpdateContext: resourceImageUpdate,
		DeleteContext: resourceImageDelete,

		Importer: &schema.ResourceImporter{
			StateContext: schema.ImportStatePassthroughContext,
		},

		Timeouts: &schema.ResourceTimeout{
			Create:  &constants.Timeout600s,
			Read:    &constants.Timeout300s,
			Update:  &constants.Timeout300s,
			Delete:  &constants.Timeout300s,
			Default: &constants.Timeout300s,
		},

		Schema: resourceImageSchemaMake(),
	}
}
